# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2024/1/12 22:05
Desc: HTTP 模式主文件
"""
import json
import logging
import urllib.parse
from logging.handlers import TimedRotatingFileHandler

import akshare as ak
from fastapi import APIRouter
from fastapi import Depends, status
from fastapi import Request
from fastapi.responses import JSONResponse, HTMLResponse
from fastapi.templating import Jinja2Templates

from byteapp_akt.datasets import get_pyscript_html, get_template_path
from byteapp_akt.login.user_login import User, get_current_active_user

app_core = APIRouter()

# 创建一个日志记录器
logger = logging.getLogger(name='AKToolsLog')
logger.setLevel(logging.INFO)

# 创建一个TimedRotatingFileHandler来进行日志轮转
handler = TimedRotatingFileHandler(
    filename='byteapp_akt_log.log', when='midnight', interval=1, backupCount=7, encoding='utf-8'
)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

# 使用日志记录器记录信息
logger.info('这是一个信息级别的日志消息')


@app_core.get("/private/{item_id}", description="私人接口", summary="该接口主要提供私密访问来获取数据")
def root(
        request: Request,
        item_id: str,
        current_user: User = Depends(get_current_active_user),
):
    """
    接收请求参数及接口名称并返回 JSON 数据
    此处由于 AKShare 的请求中是同步模式，所以这边在定义 root 函数中没有使用 asyncio 来定义，这样可以开启多线程访问
    :param request: 请求信息
    :type request: Request
    :param item_id: 必选参数; 测试接口名 ak.stock_dxsyl_em() 来获取 打新收益率 数据
    :type item_id: str
    :param current_user: 依赖注入，为了进行用户的登录验证
    :type current_user: str
    :return: 指定 接口名称 和 参数 的数据
    :rtype: json
    """
    interface_list = dir(ak)
    decode_params = urllib.parse.unquote(str(request.query_params))
    # print(decode_params)
    if item_id not in interface_list:
        return JSONResponse(
            status_code=status.HTTP_404_NOT_FOUND,
            content={
                "error": "未找到该接口，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz"
            },
        )
    eval_str = decode_params.replace("&", '", ').replace("=", '="') + '"'
    if not bool(request.query_params):
        try:
            received_df = eval("ak." + item_id + "()")
            if received_df is None:
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://akshare.akfamily.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz"
                },
            )
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))
    else:
        try:
            received_df = eval("ak." + item_id + f"({eval_str})")
            if received_df is None:
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://akshare.akfamily.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz"
                },
            )
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))


@app_core.get(path="/public/{item_id}", description="公开接口", summary="该接口主要提供公开访问来获取数据")
def root(request: Request, item_id: str):
    """
    接收请求参数及接口名称并返回 JSON 数据
    此处由于 AKShare 的请求中是同步模式，所以这边在定义 root 函数中没有使用 asyncio 来定义，这样可以开启多线程访问
    :param request: 请求信息
    :type request: Request
    :param item_id: 必选参数; 测试接口名 stock_dxsyl_em 来获取 打新收益率 数据
    :type item_id: str
    :return: 指定 接口名称 和 参数 的数据
    :rtype: json
    """
    interface_list = dir(ak)
    decode_params = urllib.parse.unquote(str(request.query_params))
    # print(decode_params)
    if item_id not in interface_list:
        logger.info("未找到该接口，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz")
        return JSONResponse(
            status_code=status.HTTP_404_NOT_FOUND,
            content={
                "error": "未找到该接口，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz"
            },
        )
    if "cookie" in decode_params:
        eval_str = (
                decode_params.split(sep="=", maxsplit=1)[0]
                + "='"
                + decode_params.split(sep="=", maxsplit=1)[1]
                + "'"
        )
        eval_str = eval_str.replace("+", " ")
    else:
        eval_str = decode_params.replace("&", '", ').replace("=", '="') + '"'
        eval_str = eval_str.replace("+", " ")  # 处理传递的参数中带空格的情况
    if not bool(request.query_params):
        try:
            received_df = eval("ak." + item_id + "()")
            if received_df is None:
                logger.info("该接口返回数据为空，请确认参数是否正确：https://akshare.akfamily.xyz")
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://akshare.akfamily.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            logger.info(
                f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz")
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz"
                },
            )
        logger.info(f"获取到 {item_id} 的数据")
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))
    else:
        try:
            received_df = eval("ak." + item_id + f"({eval_str})")
            if received_df is None:
                logger.info("该接口返回数据为空，请确认参数是否正确：https://akshare.akfamily.xyz")
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://akshare.akfamily.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            logger.info(
                f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz")
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://akshare.akfamily.xyz"
                },
            )
        logger.info(f"获取到 {item_id} 的数据")
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))


def generate_html_response():
    file_path = get_pyscript_html(file="akscript.html")
    with open(file_path, encoding="utf8") as f:
        html_content = f.read()
    return HTMLResponse(content=html_content, status_code=200)


short_path = get_template_path()
templates = Jinja2Templates(directory=short_path)


@app_core.get(
    path="/show-temp/{interface}",
    response_class=HTMLResponse,
    description="展示 PyScript",
    summary="该接口主要展示 PyScript 游览器运行 Python 代码",
)
def akscript_temp(request: Request, interface: str):
    return templates.TemplateResponse(
        "akscript.html",
        context={
            "request": request,
            "ip": request.headers["host"],
            "interface": interface,
        },
    )


@app_core.get(
    path="/show",
    response_class=HTMLResponse,
    description="展示 PyScript",
    summary="该接口主要展示 PyScript 游览器运行 Python 代码",
)
def akscript():
    return generate_html_response()



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/5/9 18:08
Desc: 导入文件工具，可以正确处理路径问题
"""
import pathlib
from importlib import resources


def get_pyscript_html(file: str = "akscript.html") -> pathlib.Path:
    """Get path to data "ths.js" text file."""
    with resources.path("byteapp_akt.assets.html", file) as f:
        data_file_path = f
    return data_file_path


def get_template_path():
    with resources.path("byteapp_akt.assets.html", "akscript.html") as f:
        data_file_path = f
    return data_file_path.parent


def get_homepage_html(file: str = "homepage.html") -> pathlib.Path:
    """Get path to data "ths.js" text file."""
    with resources.path("byteapp_akt.assets.html", file) as f:
        data_file_path = f
    return data_file_path


def get_favicon_path(file: str = "favicon.ico"):
    with resources.path("byteapp_akt.assets.images", file) as f:
        data_file_path = f
    return data_file_path


if __name__ == "__main__":
    get_pyscript_html_path = get_pyscript_html(file="akscript.html")
    print(get_pyscript_html_path)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/10/2 20:20
Desc: 配置文件
此处需要配置为 SQLite 数据库
"""
from functools import lru_cache
import logging

from dotenv import load_dotenv
from pydantic import BaseSettings

load_dotenv()

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
                    datefmt='%a, %d %b %Y %H:%M:%S',
                    filename='../log/msscore.log',
                    filemode='a+')

log = logging.getLogger("uvicorn")


class Settings(BaseSettings):
    """
    数据库配置模型类
    """
    env_name: str = "Local"

    class Config:
        env_file = "../.env"


class AuthSettings(Settings):
    """
    授权配置类
    """
    secret_key: str = "09d25e094faa6ca2556c818166b7a9563b93f7099f6f0f4caa6cf63b88e8d3e7"
    algorithm: str = "HS256"
    access_token_expire_minutes: int = 30


class LocalSettings(Settings):
    mysql_db: str = "score"
    mysql_user: str = "root"
    mysql_passwd: str = "king"
    mysql_host: str = "127.0.0.1"
    mysql_port: int = 3306


class ServerSettings(Settings):
    env_name: str = "Server"
    base_url: str = "1.1.1.1:8000"
    db_url: str = "mysql+pymysql://root@king"

    class Config:
        env_file = "../.env"


@lru_cache
def get_local_settings() -> Settings:
    settings = LocalSettings()
    log.info(f"Loading local settings for: {settings.env_name}")
    return settings


@lru_cache()
def get_auth_settings() -> Settings:
    settings = AuthSettings()
    log.info(f"加载授权环境变量成功")
    return settings



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/9/8 15:20
Desc: 数据库配置
"""
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from byteapp_akt.config import get_local_settings

local_settings = get_local_settings()

MYSQL_DB = local_settings.mysql_db  # MySQL 数据库名称
MYSQL_USER = local_settings.mysql_user  # MySQL 数据库登录帐号
MYSQL_PASSWD = local_settings.mysql_passwd  # MySQL 数据库登陆密码，由于格式化路径的问题，请勿使用 `@` 符号
MYSQL_HOST = local_settings.mysql_host  # MySQL 数据库地址，此处需要替换为服务器的地址
MYSQL_PORT = local_settings.mysql_port  # MySQL 端口
SQLALCHEMY_DATABASE_URL = f"mysql://{MYSQL_USER}:{MYSQL_PASSWD}@{MYSQL_HOST}:{MYSQL_PORT}/{MYSQL_DB}?charset=utf8mb4"
engine = create_engine(SQLALCHEMY_DATABASE_URL)
SessionLocal = sessionmaker(bind=engine, autocommit=False, autoflush=False)



"""AKTools 是 AKShare 的 HTTP API 工具, 主要目的是使 AKShare 的数据接口部署到本地从而通过 HTTP 访问来获取所需要的数据"""

"""
Changelog

0.0.1: for test
0.0.2: FastAPI test
0.0.3: FastAPI deployment test
0.0.4: add docs
0.0.5: add automate package management
0.0.6: add automate package management success
0.0.7: add automate release test
0.0.8: add automate release test again
0.0.9: add automate success
0.0.10: fix: rename workflows
0.0.11: fix: README.md
0.0.12: fix: README.md
0.0.13: fix: test pull_request
0.0.14: fix: test master push tag
0.0.15: fix: update PYPI info
0.0.16: fix: update PYPI info
0.0.17: fix: update PYPI info
0.0.18: add: add params to interface
0.0.19: fix: rename file name
0.0.20: fix: rename api name
0.0.21: fix: rename api name
0.0.22: fix: fix core module
0.0.23: fix: fix core module
0.0.24: fix: fix chinese character params in api
0.0.25: test: test
0.0.26: test: test
0.0.27: test: test
0.0.28: test: test
0.0.29: test: test
0.0.30: add: add cmd arg parser
0.0.31: add: add cmd arg parser -d
0.0.32: fix: fix docs
0.0.33: fix: fix return result
0.0.34: fix: fix return None
0.0.35: add: add multiple threads support for multiple requests
0.0.36: fix: update PYPI info
0.0.37: fix: update workflow info
0.0.38: fix: fix docs
0.0.39: fix: fix docs
0.0.40: fix: fix docs
0.0.41: fix: fix docs
0.0.42: fix: fix email
0.0.43: add: add support response status code
0.0.44: add: add login support
0.0.45: add: add module format support
0.0.46: fix: fix setup.py
0.0.47: add: add tips
0.0.48: fix: fix docs
0.0.49: add: add FastAPI docs introduction
0.0.50: add: add interface docs
0.0.51: add: add CORS support
0.0.52: add: add PyScript demo
0.0.53: fix: fix PyScript demo url
0.0.54: fix: fix PyScript demo url
0.0.55: add: add datasets.py
0.0.56: add: add init.py
0.0.57: fix: fix setup.py
0.0.58: add: add jinja2 to setup.py
0.0.59: add: add templates
0.0.60: add: add css and html to akscript.html
0.0.61: fix: fix remove dependency from akscript.html
0.0.62: add: add favicon.ico
0.0.63: fix: fix favicon.ico path
0.0.64: fix: fix favicon.ico path
0.0.65: fix: fix favicon.ico path
0.0.66: fix: fix setup.py
0.0.67: fix: fix parameter with blank character
0.0.68: fix: fix cookie in parameter
0.0.69: fix: fix file path in datasets.py
0.0.70: add: add homepage
0.0.71: add: add more info to homepage
0.0.72: add: add type hint for CLI
0.0.73: fix: fix tips
0.0.74: add: add typer for CLI
0.0.75: add: add version interface
0.0.76: fix: fix rename master to main
0.0.77: add: add cli.py file
0.0.78: add: add test_cli.py
0.0.79: fix: fix typos in homepage.html
0.0.80: fix: fix uvicorn run command
0.0.81: add: add support for Python 3.11
0.0.82: fix: fix connection.py
0.0.83: fix: fix Dockerfile
0.0.84: fix: fix homepage url
0.0.85: fix: update python version
0.0.86: add: add log function
0.0.87: fix: fix update python 3.13
"""

__title__ = "AKTools"
__version__ = "0.0.87"
__author__ = "AKFamily"



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/9/27 19:05
Desc: CLI 命令文件
"""
from pathlib import Path
from subprocess import run
from typing import Optional

import akshare as ak
import typer

import byteapp_akt

app = typer.Typer()


def version_callback(value: bool):
    if value:
        typer.echo(f"{byteapp_akt.__title__} v{byteapp_akt.__version__}")
        raise typer.Exit()


@app.command()
def main(
    host: str = typer.Option("127.0.0.1", "--host", "-H", help="设置主机地址"),
    port: int = typer.Option(
        8080, "--port", "-P", min=0, max=65535, clamp=True, help="设置端口"
    ),
    auto: bool = typer.Option(False, "--auto", "-A", help="自动打开游览器，默认为不打开"),
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="查看 AKTools 的版本",
        callback=version_callback,
        is_eager=True,
    ),
) -> None:
    app_dir = Path(__file__).parent
    order_str = (
        f"python -m uvicorn main:app --host {host} --port {port} --app-dir {app_dir}"
    )
    typer.echo(
        f"请访问：http://{host}:{port}/version 来获取最新的库版本信息，确保使用最新版本的 AKShare 和 AKTools"
    )
    typer.echo(f"当前的 AKTools 版本为：{byteapp_akt.__version__}，AKShare 版本为：{ak.__version__}")
    typer.echo(f"点击打开 HTTP API 主页：http://{host}:{port}/")
    typer.echo(f"点击打开接口导览：http://{host}:{port}/docs")
    if auto:
        typer.launch(f"http://{host}:{port}/")
    run(order_str, shell=True)


if __name__ == "__main__":
    typer.run(main)
